@@ -17,9 +17,9 @@ function(libname, pkgname)
   # Added by Uwe Ligges.
  if(.Platform$OS.type == "windows"){
      temp <- Sys.getenv("PATH")
-     Sys.putenv("PATH" = paste(normalizePath(file.path(libname, pkgname, "libs")), 
+     Sys.setenv("PATH" = paste(normalizePath(file.path(libname, pkgname, "libs")), 
                                file.path(Sys.getenv("R_HOME"), "modules", fsep="\\"), temp, sep=";"))
-     on.exit(Sys.putenv(PATH = temp))
+     on.exit(Sys.setenv(PATH = temp))
  }     
  library.dynam("XML", pkgname, libname)
 
